#!/usr/bin/env python3
import re
import subprocess
from pathlib import Path


FILES = [
    "README.md",
    "guide/src/local_development.md",
    "guide/src/tutorial.md",
    "guide/src/distribution.md",
]


def main():
    root = Path(subprocess.check_output(["git", "rev-parse", "--show-toplevel"], text=True).strip())

    for path in FILES:
        content = root.joinpath(path).read_text()

        matcher = re.compile(r"```\nUsage: maturin (\w+) (.*?)```", re.MULTILINE | re.DOTALL)

        replaces = {}
        for command, old in matcher.findall(content):
            command_output = subprocess.check_output(["cargo", "run", "--", command.lower(), "--help"], text=True)
            new = "Usage:" + command_output.strip().split("Usage:")[1] + "\n"
            # Remove trailing whitespace
            new = re.sub(" +\n", "\n", new)
            old = "Usage: maturin " + command + " " + old
            replaces[old] = new

        for old, new in replaces.items():
            content = content.replace(old, new)
        root.joinpath(path).write_text(content)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import json
import os.path
import platform
import sys
from pathlib import Path
from subprocess import check_output

from boltons.strutils import slugify

import pyo3_mixed

assert pyo3_mixed.get_42() == 42
assert slugify("First post! Hi!!!!~1    ") == "first_post_hi_1"

script_name = "print_cli_args"
args = ["a", "b", "c"]
[rust_args, python_args] = check_output([script_name, *args], text=True).splitlines()
# The rust vec debug format is also valid json
rust_args = json.loads(rust_args)
python_args = json.loads(python_args)

# On alpine/musl, rust_args is empty so we skip all tests on musl
if len(rust_args) > 0:
    # On linux we get sys.executable, windows resolve the path and mac os gives us a third
    # path (
    # {prefix}/Python.framework/Versions/3.10/Resources/Python.app/Contents/MacOS/Python
    # vs
    # {prefix}/Python.framework/Versions/3.10/bin/python3.10
    # on cirrus ci)
    # On windows, cpython resolves while pypy doesn't.
    # The script for cpython is actually a distinct file from the system interpreter for
    # windows and mac
    if platform.system() == "Linux":
        assert os.path.samefile(rust_args[0], sys.executable), (
            rust_args,
            sys.executable,
            os.path.realpath(rust_args[0]),
            os.path.realpath(sys.executable),
        )

    # Windows can't decide if it's with or without .exe, FreeBSB just doesn't work for some reason
    if platform.system() in ["Darwin", "Linux"]:
        # Unix venv layout (and hopefully also on more exotic platforms)
        print_cli_args = str(Path(sys.prefix).joinpath("bin").joinpath(script_name))
        assert rust_args[1] == print_cli_args, (rust_args, print_cli_args)
        assert python_args[0] == print_cli_args, (python_args, print_cli_args)

    # FreeBSB just doesn't work for some reason
    if platform.system() in ["Darwin", "Linux", "Windows"]:
        # Rust contains the python executable as first argument but python does not
        assert rust_args[2:] == args, rust_args
        assert python_args[1:] == args, python_args

print("SUCCESS")



import os
import platform
import sys
from subprocess import check_output


def main():
    if platform.system().lower() == "windows":
        # Add sys.base_prefix which contains python3y.dll to PATH
        # otherwise running `pyo3-bin` might return exit code 3221225781
        path = os.environ["PATH"]
        path = path + os.pathsep + sys.base_prefix
        os.environ["PATH"] = path

    output = check_output(["pyo3-bin"]).decode("utf-8").strip()
    if not output == "Hello, world!":
        raise Exception(output)
    print("SUCCESS")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import pyo3_mixed_with_path_dep

assert pyo3_mixed_with_path_dep.get_42() == 42, "get_42 did not return 42"

assert pyo3_mixed_with_path_dep.is_half(21, 42), "21 is not half of 42"
assert not pyo3_mixed_with_path_dep.is_half(21, 73), "21 is half of 63"

print("SUCCESS")



#!/usr/bin/env python3

import pyo3_pure

import pytest


def test_static():
    assert pyo3_pure.fourtytwo == 42


def test_class():
    assert pyo3_pure.DummyClass.get_42() == 42


def test_function():
    with pytest.raises(AssertionError):
        assert pyo3_pure.DummyClass == 42



#!/usr/bin/env python3

import cffi_mixed

from cffi_mixed import Line

point = cffi_mixed.lib.get_origin()
point.x = 10
point.y = 10
assert cffi_mixed.lib.is_in_range(point, 15)

assert Line(2, 5, 6, 8).length() == 5

print("SUCCESS")



import math

from .cffi_mixed import ffi


class Line:
    def __init__(self, x1: float, y1: float, x2: float, y2: float):
        # You can pass a tuple/list or a dict as value for a public rust struct
        self.start = ffi.new("Point *", {"x": x1, "y": y1})
        self.end = ffi.new("Point *", (x2, y2))

    def length(self) -> float:
        """Returns the length of the line."""
        return math.sqrt((self.end.x - self.start.x) ** 2 + (self.end.y - self.start.y) ** 2)

    def __str__(self) -> str:
        return "Line from ({},{}) to ({},{})".format(self.start.x, self.start.y, self.end.x, self.end.y)



import locale
import sys
from pathlib import Path

import with_data

assert with_data.lib.one() == 1
assert with_data.ffi.string(with_data.lib.say_hello()).decode() == "hello"

venv_root = Path(sys.prefix)

installed_data = (
    venv_root.joinpath("data_subdir")
    .joinpath("hello.txt")
    # With the default encoding, python under windows fails to read the file correctly
    .read_text(encoding="utf-8")
    .strip()
)
assert installed_data == "Hi! 😊", (
    installed_data,
    "Hi! 😊",
    locale.getpreferredencoding(),
)
header_file = (
    venv_root.joinpath("include")
    .joinpath("site")
    .joinpath(f"python{sys.version_info.major}.{sys.version_info.minor}")
    .joinpath("with-data")
    .joinpath("empty.h")
)
assert header_file.is_file(), header_file

print("SUCCESS")



#!/usr/bin/env python3

import pyo3_mixed_implicit.some_rust as some_rust


def test_get_rust_and_python():
    assert some_rust.get_22() == 22
    assert some_rust.double(lambda: 4) == 8


print("SUCCESS")



#!/usr/bin/env python3

import cffi_pure


def test_range():
    point = cffi_pure.lib.get_origin()
    point.x = 10
    point.y = 10

    assert not cffi_pure.lib.is_in_range(point, 14)
    assert cffi_pure.lib.is_in_range(point, 15)


def test_ffi():
    point = cffi_pure.ffi.new("Point *", (10, 20))
    assert point.x == 10
    assert point.y == 20



from subprocess import check_output


def main():
    output = check_output(["license-test"]).decode("utf-8").strip()
    if not output == "Hello, world!":
        raise Exception(output)
    print("SUCCESS")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import cffi_mixed

from cffi_mixed import Line


def test_in_range():
    point = cffi_mixed.lib.get_origin()
    point.x = 10
    point.y = 10

    assert not cffi_mixed.lib.is_in_range(point, 14)
    assert cffi_mixed.lib.is_in_range(point, 15)


def test_ffi():
    point = cffi_mixed.ffi.new("Point *", (10, 20))
    assert point.x == 10
    assert point.y == 20


def test_line():
    line = Line(2, 5, 6, 8)
    assert line.length() == 5
    assert str(line) == "Line from (2.0,5.0) to (6.0,8.0)"



# maturin is self bootstrapping, however on platforms like FreeBSD that aren't
# manylinux/musllinux, pip will try installing maturin from the source distribution.
# That source distribution obviously can't depend on maturin, so we're using
# the always available setuptools.
#
# Note that this is really only a workaround for bootstrapping and not suited
# for general purpose packaging, i.e. only building a wheel (as in
# `python setup.py bdist_wheel`) and installing (as in
# `pip install <source dir>`) are supported. For creating a source distribution
# for maturin itself use `maturin sdist`.

import os
import shlex

try:
    import tomllib
except ModuleNotFoundError:
    import tomli as tomllib
from setuptools import setup

from setuptools_rust import RustBin

# Force the wheel to be platform specific
# https://stackoverflow.com/a/45150383/3549270
# There's also the much more concise solution in
# https://stackoverflow.com/a/53463910/3549270,
# but that would require python-dev
try:
    # noinspection PyPackageRequirements,PyUnresolvedReferences
    from wheel.bdist_wheel import bdist_wheel as _bdist_wheel

    # noinspection PyPep8Naming,PyAttributeOutsideInit
    class bdist_wheel(_bdist_wheel):
        def finalize_options(self):
            _bdist_wheel.finalize_options(self)
            self.root_is_pure = False

except ImportError:
    bdist_wheel = None

with open("Cargo.toml", "rb") as fp:
    version = tomllib.load(fp)["package"]["version"]

# Use `--no-default-features` by default for a minimal build to support PEP 517.
# `MATURIN_SETUP_ARGS` env var can be used to pass customized arguments to cargo.
cargo_args = ["--no-default-features"]
if os.getenv("MATURIN_SETUP_ARGS"):
    cargo_args = shlex.split(os.getenv("MATURIN_SETUP_ARGS", ""))

setup(
    version=version,
    cmdclass={"bdist_wheel": bdist_wheel},
    rust_extensions=[RustBin("maturin", args=cargo_args, cargo_manifest_args=["--locked"])],
    zip_safe=False,
)



import os
import json
import sys
from pathlib import Path

import nox


PYODIDE_VERSION = os.getenv("PYODIDE_VERSION", "0.23.4")
GITHUB_ACTIONS = os.getenv("GITHUB_ACTIONS")
GITHUB_ENV = os.getenv("GITHUB_ENV")
MSRV = "1.74.0"


def append_to_github_env(name: str, value: str):
    if not GITHUB_ACTIONS or not GITHUB_ENV:
        return

    with open(GITHUB_ENV, "w+") as f:
        f.write(f"{name}={value}\n")


@nox.session(name="update-pyo3", python=False)
def update_pyo3(session: nox.Session):
    # TODO: support updating major and minor versions by editing Cargo.toml first
    test_crate_dir = Path("./test-crates").resolve()
    crates_to_update = ["pyo3", "pyo3-ffi", "python3-dll-a"]
    for root, _, files in os.walk(test_crate_dir):
        if "Cargo.lock" in files:
            cargo_lock_path = Path(root) / "Cargo.lock"
            with open(cargo_lock_path, "r") as lock_file:
                content = lock_file.read()
                for crate in crates_to_update:
                    if f'name = "{crate}"' in content:
                        with session.chdir(root):
                            session.run("cargo", f"+{MSRV}", "update", "-p", crate, external=True)


@nox.session(name="setup-pyodide", python=False)
def setup_pyodide(session: nox.Session):
    tests_dir = Path("./tests").resolve()
    with session.chdir(tests_dir):
        session.run(
            "npm",
            "i",
            "--no-save",
            f"pyodide@{PYODIDE_VERSION}",
            "prettier",
            external=True,
        )
        with session.chdir(tests_dir / "node_modules" / "pyodide"):
            session.run(
                "node",
                "../prettier/bin/prettier.cjs",
                "-w",
                "pyodide.asm.js",
                external=True,
            )
            with open("repodata.json") as f:
                emscripten_version = json.load(f)["info"]["platform"].split("_", 1)[1].replace("_", ".")
                append_to_github_env("EMSCRIPTEN_VERSION", emscripten_version)


@nox.session(name="test-emscripten", python=False)
def test_emscripten(session: nox.Session):
    tests_dir = Path("./tests").resolve()

    test_crates = [
        "test-crates/pyo3-pure",
        "test-crates/pyo3-mixed",
    ]
    for crate in test_crates:
        crate = Path(crate).resolve()
        ver = sys.version_info
        session.run("cargo", "build", external=True)
        session.run(
            tests_dir.parent / "target" / "debug" / "maturin",
            "build",
            "-m",
            str(crate / "Cargo.toml"),
            "--target",
            "wasm32-unknown-emscripten",
            "-i",
            f"python{ver.major}.{ver.minor}",
            env={"RUSTUP_TOOLCHAIN": "nightly"},
            external=True,
        )

        with session.chdir(tests_dir):
            session.run("node", "emscripten_runner.js", str(crate), external=True)



#!/usr/bin/env python3
import os
import subprocess

import pyo3_pure

assert pyo3_pure.DummyClass.get_42() == 42

# Check type stub
install_path = os.path.join(os.path.dirname(pyo3_pure.__file__))
assert os.path.exists(os.path.join(install_path, "__init__.pyi"))
assert os.path.exists(os.path.join(install_path, "py.typed"))

# Check entrypoints
assert subprocess.run(["get_42"]).returncode == 42

print("SUCCESS")



import json
import platform
import sys
import sysconfig
import struct

if platform.python_implementation() == "PyPy":
    # Workaround for PyPy 3.6 on windows:
    #  - sysconfig.get_config_var("EXT_SUFFIX") differs to importlib until
    #    Python 3.8
    #  - PyPy does not load the plain ".pyd" suffix because it expects that's
    #    for a CPython extension module
    #
    # This workaround can probably be removed once PyPy for Python 3.8 is the
    # main PyPy version.
    import importlib.machinery

    ext_suffix = importlib.machinery.EXTENSION_SUFFIXES[0]
else:
    ext_suffix = sysconfig.get_config_var("EXT_SUFFIX")

metadata = {
    # sys.implementation.name can differ from platform.python_implementation(), for example
    # Pyston has sys.implementation.name == "pyston" while platform.python_implementation() == cpython
    "implementation_name": sys.implementation.name,
    "executable": sys.executable or None,
    "major": sys.version_info.major,
    "minor": sys.version_info.minor,
    "abiflags": sysconfig.get_config_var("ABIFLAGS"),
    "interpreter": platform.python_implementation().lower(),
    "ext_suffix": ext_suffix,
    "soabi": sysconfig.get_config_var("SOABI") or None,
    "platform": sysconfig.get_platform(),
    # This one isn't technically necessary, but still very useful for sanity checks
    "system": platform.system().lower(),
    # This one is for generating a config file for pyo3
    "pointer_width": struct.calcsize("P") * 8,
    "gil_disabled": sysconfig.get_config_var("Py_GIL_DISABLED") == 1,
}

print(json.dumps(metadata))
